@@ -259,9 +259,9 @@ class JumpTableProcessor(
         offset = stmt.offset
         data = self._expr(stmt.data)
         if self._tsrc is not None:
-            r = [self._tsrc, data]
+            r = (self._tsrc, data)
         else:
-            r = [(self.block.addr, self.stmt_idx), data]
+            r = ((self.block.addr, self.stmt_idx), data)
         self.state._registers[offset] = r
 
     def _handle_Store(self, stmt):
@@ -343,7 +343,7 @@ class JumpTableProcessor(
         self._handle_Comparison(*expr.args)
 
     def _handle_CmpGT(self, expr):
-        self._handle_CmpGE(*expr.args)
+        self._handle_Comparison(*expr.args)
 
     def _handle_CCall(self, expr):
         if not isinstance(expr.args[0], pyvex.IRExpr.Const):
@@ -660,7 +660,7 @@ class JumpTableResolver(IndirectJumpResolver):
 
         self._max_targets = cfg._indirect_jump_target_limit
 
-        for slice_steps in range(1, 4):
+        for slice_steps in range(1, 5):
             # Perform a backward slicing from the jump target
             # Important: Do not go across function call boundaries
             b = Blade(cfg.graph, addr, -1,
@@ -837,6 +837,18 @@ class JumpTableResolver(IndirectJumpResolver):
 
                 assert ret is not None
 
+                # finally, we filter jump targets according to the alignment of the architecture
+                if is_arm_arch(self.project.arch):
+                    alignment = 4 if addr % 2 == 0 else 2
+                else:
+                    alignment = self.project.arch.instruction_alignment
+                if alignment != 1:
+                    if is_arm_arch(self.project.arch) and addr % 2 == 1:
+                        # Special logic for handling THUMB addresses
+                        all_targets = [t_ for t_ in all_targets if (t_ - 1) % alignment == 0]
+                    else:
+                        all_targets = [ t_ for t_ in all_targets if t_ % alignment == 0 ]
+
                 l.info("Resolved %d targets from %#x.", len(all_targets), addr)
 
                 # write to the IndirectJump object in CFG
