@@ -20,6 +20,9 @@ l = logging.getLogger(name=__name__)
 
 
 class OverwriteTmpValueCallback:
+    """
+    Overwrites temporary values during resolution
+    """
     def __init__(self, gp_value):
         self.gp_value = gp_value
 
@@ -28,8 +31,11 @@ class OverwriteTmpValueCallback:
 
 
 class MipsElfFastResolver(IndirectJumpResolver):
+    """
+    Indirect Jump Resolver for MIPs
+    """
     def __init__(self, project):
-        super(MipsElfFastResolver, self).__init__(project, timeless=True)
+        super().__init__(project, timeless=True)
 
     def filter(self, cfg, addr, func_addr, block, jumpkind):
         if not isinstance(self.project.arch, (archinfo.ArchMIPS32, archinfo.ArchMIPS64, )):
@@ -38,7 +44,8 @@ class MipsElfFastResolver(IndirectJumpResolver):
 
     def resolve(self, cfg, addr, func_addr, block, jumpkind):
         """
-        Resolves the indirect jump in MIPS ELF binaries where all external function calls are indexed using gp.
+        Wrapper for _resolve that slowly increments the max_depth used by Blade for finding sources
+        until we can resolve the addr or we reach the default max_depth
 
         :param cfg: A CFG instance.
         :param int addr: IRSB address.
@@ -48,11 +55,30 @@ class MipsElfFastResolver(IndirectJumpResolver):
         :return: If it was resolved and targets alongside it
         :rtype: tuple
         """
+        for max_level in range(2, 4):
+            resolved, resolved_targets = self._resolve(cfg, addr, func_addr, block, jumpkind, max_level=max_level)
+            if resolved:
+                return resolved, resolved_targets
+        return False, []
+
+    def _resolve(self, cfg, addr, func_addr, block, jumpkind, max_level):
+        """
+        Resolves the indirect jump in MIPS ELF binaries where all external function calls are indexed using gp.
+
+        :param cfg: A CFG instance.
+        :param int addr: IRSB address.
+        :param int func_addr: The function address.
+        :param pyvex.IRSB block: The IRSB.
+        :param str jumpkind: The jumpkind.
+        :param int max_level: maximum level for Blade to resolve when looking for sources
+        :return: If it was resolved and targets alongside it
+        :rtype: tuple
+        """
 
         project = self.project
 
         b = Blade(cfg.graph, addr, -1, cfg=cfg, project=project, ignore_sp=True, ignore_bp=True,
-                  ignored_regs=('gp',), cross_insn_opt=False, stop_at_calls=True
+                  ignored_regs=('gp',), cross_insn_opt=False, stop_at_calls=True, max_level=max_level
                   )
 
         sources = [n for n in b.slice.nodes() if b.slice.in_degree(n) == 0]
@@ -112,7 +138,7 @@ class MipsElfFastResolver(IndirectJumpResolver):
                 return False, [ ]
             target = target_state.addr
 
-            if self._is_target_valid(cfg, target):
+            if self._is_target_valid(cfg, target) and target != func_addr:
                 l.debug("Indirect jump at %#x is resolved to target %#x.", addr, target)
                 return True, [ target ]
 
@@ -124,7 +150,6 @@ class MipsElfFastResolver(IndirectJumpResolver):
 
     @staticmethod
     def _set_gp_load_callback(state, blade, project, gp_offset, gp_value):
-        got_gp_stack_store = False
         tmps = {}
         for block_addr_in_slice in set(slice_node[0] for slice_node in blade.slice.nodes()):
             for stmt in project.factory.block(block_addr_in_slice, cross_insn_opt=False).vex.statements:
@@ -138,16 +163,13 @@ class MipsElfFastResolver(IndirectJumpResolver):
                             # found the load from stack
                             # we must make sure value of that temporary variable equals to the correct gp value
                             state.inspect.make_breakpoint('tmp_write', when=BP_BEFORE,
-                                                          condition=lambda s, bbl_addr_=block_addr_in_slice,
-                                                                           tmp_offset_=tmp_offset:
-                                                          s.scratch.bbl_addr == bbl_addr_ and s.inspect.tmp_write_num == tmp_offset_,
-                                                          action=OverwriteTmpValueCallback(
-                                                              gp_value).overwrite_tmp_value
-                                                          )
-                            got_gp_stack_store = True
+                                        condition=lambda s, bbl_addr_=block_addr_in_slice,
+                                                        tmp_offset_=tmp_offset:
+                                        s.scratch.bbl_addr == bbl_addr_ and s.inspect.tmp_write_num == tmp_offset_,
+                                        action=OverwriteTmpValueCallback(
+                                        gp_value).overwrite_tmp_value
+                                        )
                             break
-            if got_gp_stack_store:
-                break
 
     @staticmethod
     def _is_gp_used_on_slice(project, b: Blade) -> bool:
