@@ -1,9 +1,9 @@
-# pylint:disable=unused-argument
+# pylint:disable=unused-argument,no-self-use
 from typing import Dict, Type, Callable, Any, Optional
 
 from ailment import Block
 from ailment.statement import Call, Statement, ConditionalJump, Assignment, Store, Return
-from ailment.expression import Load, Expression, BinaryOp, UnaryOp, Convert, ITE
+from ailment.expression import Load, Expression, BinaryOp, UnaryOp, Convert, ITE, DirtyExpression, VEXCCallExpression
 
 
 class AILBlockWalker:
@@ -27,6 +27,8 @@ class AILBlockWalker:
             UnaryOp: self._handle_UnaryOp,
             Convert: self._handle_Convert,
             ITE: self._handle_ITE,
+            DirtyExpression: self._handle_DirtyExpression,
+            VEXCCallExpression: self._handle_VEXCCallExpression,
         }
 
         self.stmt_handlers: Dict[Type, Callable] = stmt_handlers if stmt_handlers else _default_stmt_handlers
@@ -40,10 +42,11 @@ class AILBlockWalker:
             i += 1
 
     def walk_statement(self, stmt: Statement):
-        self._handle_stmt(0, stmt, None)
+        return self._handle_stmt(0, stmt, None)
 
-    def walk_expression(self, expr: Expression):
-        self._handle_expr(0, expr, 0, None, None)
+    def walk_expression(self, expr: Expression, stmt_idx: Optional[int]=None, stmt: Optional[int]=None,
+                        block: Optional[Block]=None):
+        return self._handle_expr(0, expr, stmt_idx, stmt, block)
 
     def _handle_stmt(self, stmt_idx: int, stmt: Statement, block: Optional[Block]) -> Any:
         try:
@@ -67,7 +70,7 @@ class AILBlockWalker:
             if expr is not None:
                 r = self._handle_expr(expr_idx, expr, stmt_idx, stmt, block)
                 return r if r is not None else expr
-        return None
+        return None  # unchanged
 
     #
     # Default handlers
@@ -274,3 +277,30 @@ class AILBlockWalker:
             new_expr.iffalse = iffalse
             return new_expr
         return None
+
+    def _handle_DirtyExpression(self, expr_idx: int, expr: DirtyExpression, stmt_idx: int, stmt: Statement,
+                                block: Optional[Block]):
+        new_dirty_expr = self._handle_expr(0, expr.dirty_expr, stmt_idx, stmt, block)
+        if new_dirty_expr is not None and new_dirty_expr is not expr.dirty_expr:
+            new_expr = expr.copy()
+            new_expr.dirty_expr = new_dirty_expr
+            return new_expr
+        return None
+
+    def _handle_VEXCCallExpression(self, expr_idx: int, expr: VEXCCallExpression, stmt_idx: int, stmt: Statement,
+                                   block: Optional[Block]):
+        changed = False
+        new_operands = [ ]
+        for idx, operand in enumerate(expr.operands):
+            new_operand = self._handle_expr(idx, operand, stmt_idx, stmt, block)
+            if new_operand is not None and new_operand is not operand:
+                changed = True
+                new_operands.append(new_operand)
+            else:
+                new_operands.append(operand)
+
+        if changed:
+            new_expr = expr.copy()
+            new_expr.operands = tuple(new_operands)
+            return new_expr
+        return None
