@@ -19,6 +19,9 @@ class EmptyNodeRemover:
     def __init__(self, node):
         self.root = node
 
+        self.removed_sequences = [ ]
+        self.replaced_sequences = { }
+
         handlers = {
             SequenceNode: self._handle_Sequence,
             CodeNode: self._handle_Code,
@@ -36,11 +39,11 @@ class EmptyNodeRemover:
         self._walker = SequenceWalker(handlers=handlers)
         r = self._walker.walk(self.root)
         if r is None:
-            self.result = SequenceNode(nodes=[])
+            self.result = SequenceNode(None, nodes=[])
         else:
             # Make sure it's still a sequence node
             if not isinstance(r, SequenceNode):
-                r = SequenceNode(nodes=[r])
+                r = SequenceNode(node.addr, nodes=[r])
             self.result = r
 
     #
@@ -59,11 +62,16 @@ class EmptyNodeRemover:
                     new_nodes.append(new_node)
 
         if not new_nodes:
+            self.removed_sequences.append(node)
             return None
         if len(new_nodes) == 1:
             # Remove the unnecessary sequence node
+            self.replaced_sequences[node] = new_nodes[0]
             return new_nodes[0]
-        return SequenceNode(nodes=new_nodes)
+
+        sn = SequenceNode(node.addr, nodes=new_nodes)
+        self.replaced_sequences[node] = sn
+        return sn
 
     def _handle_MultiNode(self, node: MultiNode, **kwargs):
 
@@ -90,6 +98,9 @@ class EmptyNodeRemover:
         if claripy.is_true(node.reaching_condition):
             # Remove the unnecessary CodeNode
             return inner_node
+        if isinstance(inner_node, CodeNode):
+            # unpack the codenode so we don't have directly nested CodeNodes
+            return CodeNode(inner_node.node, claripy.And(node.reaching_condition, inner_node.reaching_condition))
         return CodeNode(inner_node, node.reaching_condition)
 
     def _handle_Condition(self, node, **kwargs):
