@@ -1,3 +1,4 @@
+# pylint:disable=unused-argument
 from typing import TYPE_CHECKING
 
 import claripy
@@ -16,6 +17,9 @@ class SimEngineVRVEX(
     SimEngineLightVEXMixin,
     SimEngineVRBase,
 ):
+    """
+    Implements the VEX engine for variable recovery analysis.
+    """
     state: 'VariableRecoveryStateBase'
 
     # Statement handlers
@@ -120,7 +124,8 @@ class SimEngineVRVEX(
         return None
 
     def _handle_Const(self, expr):
-        return RichR(claripy_value(expr.con.type, expr.con.value), typevar=typeconsts.int_type(expr.con.size))
+        return RichR(claripy_value(expr.con.type, expr.con.value, size=expr.con.size),
+                     typevar=typeconsts.int_type(expr.con.size))
 
     def _handle_Add(self, expr):
         arg0, arg1 = expr.args
@@ -339,6 +344,27 @@ class SimEngineVRVEX(
                      typevar=r0.typevar,
                      )
 
+    def _handle_CmpF(self, expr):
+        return RichR(self.state.top(expr.result_size(self.tyenv)))
+
+    def _handle_16HLto32(self, expr):
+        return RichR(self.state.top(32))
+
+    def _handle_Add_v(self, expr, vector_size, vector_count):
+        return RichR(self.state.top(expr.result_size(self.tyenv)))
+
+    def _handle_QSub_v(self, expr, vector_size, vector_count):
+        return RichR(self.state.top(expr.result_size(self.tyenv)))
+
+    def _handle_HAdd_v(self, expr, vector_size, vector_count):
+        return RichR(self.state.top(expr.result_size(self.tyenv)))
+
+    def _handle_Clz(self, expr):
+        return RichR(self.state.top(expr.result_size(self.tyenv)))
+
+    def _handle_Mull(self, expr):
+        return RichR(self.state.top(expr.result_size(self.tyenv)))
+
     def _handle_CmpEQ(self, expr):
         arg0, arg1 = expr.args
         _ = self._expr(arg0)
@@ -380,3 +406,13 @@ class SimEngineVRVEX(
         _ = self._expr(arg1)
 
         return RichR(self.state.top(1))
+
+    def _handle_Cmp_v(self, expr, vector_size, vector_count):
+        return RichR(self.state.top(1))
+
+    _handle_CmpEQ_v = _handle_Cmp_v
+    _handle_CmpNE_v = _handle_Cmp_v
+    _handle_CmpLE_v = _handle_Cmp_v
+    _handle_CmpLT_v = _handle_Cmp_v
+    _handle_CmpGE_v = _handle_Cmp_v
+    _handle_CmpGT_v = _handle_Cmp_v
