@@ -1,16 +1,18 @@
-from typing import Dict, Optional
+# pylint:disable=raise-missing-from
+from typing import Dict, Set, Optional
 import logging
 import collections.abc
 from sortedcontainers import SortedDict
+
 import networkx
 
+from archinfo.arch_soot import SootMethodDescriptor
+
 from ...errors import SimEngineError
 from ..plugin import KnowledgeBasePlugin
-
 from .function import Function
 from .soot_function import SootFunction
 
-from archinfo.arch_soot import SootMethodDescriptor
 
 l = logging.getLogger(name=__name__)
 
@@ -23,14 +25,14 @@ class FunctionDict(SortedDict):
     def __init__(self, backref, *args, **kwargs):
         self._backref = backref
         self._key_types = kwargs.pop('key_types', int)
-        super(FunctionDict, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def __getitem__(self, addr):
         try:
-            return super(FunctionDict, self).__getitem__(addr)
-        except KeyError:
+            return super().__getitem__(addr)
+        except KeyError as ex:
             if not isinstance(addr, self._key_types):
-                raise TypeError("FunctionDict only supports %s as key type" % self._key_types)
+                raise TypeError("FunctionDict only supports %s as key type" % self._key_types) from ex
 
             if isinstance(addr, SootMethodDescriptor):
                 t = SootFunction(self._backref, addr)
@@ -38,13 +40,13 @@ class FunctionDict(SortedDict):
                 t = Function(self._backref, addr)
             try:
                 self[addr] = t
-            except:
+            except Exception:  # pylint:disable=broad-except
                 pass
             self._backref._function_added(t)
             return t
 
     def get(self, addr):
-        return super(FunctionDict, self).__getitem__(addr)
+        return super().__getitem__(addr)
 
     def floor_addr(self, addr):
         try:
@@ -65,17 +67,37 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
     results during CFG generation, and manages a function map of the binary.
     """
     def __init__(self, kb):
-        super(FunctionManager, self).__init__()
+        super().__init__()
         self._kb = kb
         self.function_address_types = self._kb._project.arch.function_address_types
         self.address_types = self._kb._project.arch.address_types
         self._function_map: Dict[int,Function] = FunctionDict(self, key_types=self.function_address_types)
+        self.function_addrs_set: Set = set()
         self.callgraph = networkx.MultiDiGraph()
         self.block_map = {}
 
         # Registers used for passing arguments around
         self._arg_registers = kb._project.arch.argument_registers
 
+    def __setstate__(self, state):
+        self._kb = state["_kb"]
+        self.function_address_types = state["function_address_types"]
+        self.address_types = state["address_types"]
+        self._function_map = state["_function_map"]
+        self.callgraph = state["callgraph"]
+        self.block_map = state["block_map"]
+
+    def __getstate__(self):
+        s = {
+            "_kb": self._kb,
+            "function_address_types": self.function_address_types,
+            "address_types": self.address_types,
+            "_function_map": self._function_map,
+            "callgraph": self.callgraph,
+            "block_map": self.block_map,
+        }
+        return s
+
     def copy(self):
         fm = FunctionManager(self._kb)
         fm._function_map = self._function_map.copy()
@@ -83,13 +105,15 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
             fm._function_map[address] = function.copy()
         fm.callgraph = networkx.MultiDiGraph(self.callgraph)
         fm._arg_registers = self._arg_registers.copy()
+        fm.function_addrs_set = self.function_addrs_set.copy()
 
         return fm
 
     def clear(self):
-        self._function_map.clear()
+        self._function_map = FunctionDict(self, key_types=self.function_address_types)
         self.callgraph = networkx.MultiDiGraph()
         self.block_map.clear()
+        self.function_addrs_set = set()
 
     def _genenare_callmap_sif(self, filepath):
         """
@@ -111,8 +135,8 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
         dst_func._register_nodes(True, node)
         self.block_map[node.addr] = node
 
-    def _add_call_to(self, function_addr, from_node, to_addr, retn_node=None, syscall=None, stmt_idx=None, ins_addr=None,
-                     return_to_outside=False):
+    def _add_call_to(self, function_addr, from_node, to_addr, retn_node=None, syscall=None, stmt_idx=None,
+                     ins_addr=None, return_to_outside=False):
         """
         Add a call to a function.
 
@@ -268,8 +292,10 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
             del self._function_map[k]
             if k in self.callgraph:
                 self.callgraph.remove_node(k)
+            self.function_addrs_set.discard(k)
         else:
-            raise ValueError("FunctionManager.__delitem__ only accepts int as key")
+            raise ValueError(f"FunctionManager.__delitem__ only accepts the following address types: "
+                             f"{self.function_address_types}")
 
     def __len__(self):
         return len(self._function_map)
@@ -289,6 +315,9 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
         :return:       None
         """
 
+        # Add the function address to the set of function addresses
+        self.function_addrs_set.add(func.addr)
+
         # make sure all functions exist in the call graph
         self.callgraph.add_node(func.addr)
 
