@@ -13,11 +13,11 @@ _l = logging.getLogger(name=__name__)
 lib = SimLibrary()
 lib.set_default_cc('X86', SimCCStdcall)
 lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
-lib.set_library_names("api-ms-win-core-comm-l1-1-2.dll")
+lib.set_library_names("api-ms-win-core-comm-l1-1-1.dll")
 prototypes = \
     {
         # 
-        'GetCommPorts': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpPortNumbers", "uPortNumbersCount", "puPortNumbersFound"]),
+        'OpenCommPort': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["uPortNumber", "dwDesiredAccess", "dwFlagsAndAttributes"]),
     }
 
 lib.set_prototypes(prototypes)
