@@ -15,16 +15,14 @@ class snprintf(FormatParser):
         if self.state.solver.eval(size) == 0:
             return size
 
-        # The format str is at index 2
-        fmt_str = self._parse(2)
-        out_str = fmt_str.replace(3, self.arg)
+        fmt_str = self._parse(fmt)
+        out_str = fmt_str.replace(self.va_arg)
         self.state.memory.store(dst_ptr, out_str)
 
         # place the terminating null byte
-        self.state.memory.store(dst_ptr + (out_str.size() // 8), self.state.solver.BVV(0, 8))
+        self.state.memory.store(dst_ptr + (out_str.size() // self.arch.byte_width), self.state.solver.BVV(0, 8))
 
-        # size_t has size arch.bits
-        return self.state.solver.BVV(out_str.size()//8, self.state.arch.bits)
+        return out_str.size()//self.arch.byte_width
 
 ######################################
 # __snprintf_chk
@@ -35,12 +33,11 @@ class __snprintf_chk(FormatParser):
     def run(self, dst_ptr, maxlen, size, fmt):  # pylint:disable=arguments-differ,unused-argument
 
         # The format str is at index 4
-        fmt_str = self._parse(4)
-        out_str = fmt_str.replace(5, self.arg)
+        fmt_str = self._parse(fmt)
+        out_str = fmt_str.replace(self.va_arg)
         self.state.memory.store(dst_ptr, out_str)
 
         # place the terminating null byte
-        self.state.memory.store(dst_ptr + (out_str.size() // 8), self.state.solver.BVV(0, 8))
+        self.state.memory.store(dst_ptr + (out_str.size() // self.arch.byte_width), self.state.solver.BVV(0, 8))
 
-        # size_t has size arch.bits
-        return self.state.solver.BVV(out_str.size()//8, self.state.arch.bits)
+        return out_str.size()//self.arch.byte_width
