@@ -10,4 +10,4 @@ class bind(angr.SimProcedure):
     #pylint:disable=arguments-differ
 
     def run(self, fd, addr_ptr, addr_len): #pylint:disable=unused-argument
-        return self.state.solver.Unconstrained('bind', self.state.arch.bits, key=('api', 'bind'))
+        return self.state.solver.Unconstrained('bind', self.arch.sizeof['int'], key=('api', 'bind'))
