@@ -13,7 +13,12 @@ class ReturnUnconstrained(angr.SimProcedure):
 
         return_val = kwargs.pop('return_val', None)
         if return_val is None:
-            o = self.state.solver.Unconstrained("unconstrained_ret_%s" % self.display_name, self.state.arch.bits, key=('api', '?', self.display_name))
+            size = self.prototype.returnty.size
+            # ummmmm do we really want to rely on this behavior?
+            if size is NotImplemented:
+                o = None
+            else:
+                o = self.state.solver.Unconstrained("unconstrained_ret_%s" % self.display_name, size, key=('api', '?', self.display_name))
         else:
             o = return_val
 
