@@ -48,6 +48,18 @@ class SimFilesystem(SimStatePlugin): # pretends links don't exist
         for fname in files:
             self.insert(fname, files[fname])
 
+    @SimStatePlugin.memo
+    def copy(self, memo):
+        o = super().copy(memo)
+
+        o.pathsep=self.pathsep
+        o.cwd=self.cwd
+        o._unlinks = list(self._unlinks)
+        o._files={k: v.copy(memo) for k, v in self._files.items()}
+        o._mountpoints={k: v.copy(memo) for k, v in self._mountpoints.items()}
+
+        return o
+
     @property
     def unlinks(self):
         for _, f in self._unlinks:
@@ -61,17 +73,6 @@ class SimFilesystem(SimStatePlugin): # pretends links don't exist
         for fname in self._mountpoints:
             self._mountpoints[fname].set_state(state)
 
-    @SimStatePlugin.memo
-    def copy(self, memo):
-        o = SimFilesystem(
-                files={k: v.copy(memo) for k, v in self._files.items()},
-                pathsep=self.pathsep,
-                cwd=self.cwd,
-                mountpoints={k: v.copy(memo) for k, v in self._mountpoints.items()}
-            )
-        o._unlinks = list(self._unlinks)
-        return o
-
     def merge(self, others, merge_conditions, common_ancestor=None):
         for o in others:
             if o.cwd != self.cwd:
@@ -316,7 +317,7 @@ class SimConcreteFilesystem(SimMount):
         return True
 
     def delete(self, path_elements):
-        path = self.pathsep.join(x.decode() for x in path_elements)
+        path = self._join_chunks([x.decode() for x in path_elements])
         self.deleted_list.add(path)
         return self.cache.pop(path, None) is not None
 
