@@ -187,6 +187,37 @@ class SimSystemPosix(SimStatePlugin):
         self.stdout = stdout
         self.stderr = stderr
 
+    @SimStatePlugin.memo
+    def copy(self, memo):
+        o = super().copy(memo)
+
+        o.sigmask_bits = self.sigmask_bits
+        o.maximum_symbolic_syscalls = self.maximum_symbolic_syscalls
+        o.max_length = self.max_length
+        o.stdin = self.stdin.copy(memo)
+        o.stdout = self.stdout.copy(memo)
+        o.stderr = self.stderr.copy(memo)
+        o.fd = {k: self.fd[k].copy(memo) for k in self.fd}
+        o.sockets = {ident: tuple(x.copy(memo) for x in self.sockets[ident]) for ident in self.sockets}
+        o.socket_queue = self.socket_queue # shouldn't need to copy this - should be copied before use.
+        o.argv = self.argv
+        o.argc = self.argc
+        o.environ = self.environ
+        o.auxv = self.auxv
+        o.tls_modules = self.tls_modules
+        o._sigmask = self._sigmask
+        o.pid = self.pid
+        o.ppid = self.ppid
+        o.uid = self.uid
+        o.gid = self.gid
+        o.brk = self.brk
+        o.autotmp_counter = self.autotmp_counter
+        o.dev_fs = self.dev_fs.copy(memo)
+        o.proc_fs = self.proc_fs.copy(memo)
+        o._closed_fds = list(self._closed_fds)
+
+        return o
+
     @property
     def closed_fds(self):
         for _, f in self._closed_fds:
@@ -230,7 +261,14 @@ class SimSystemPosix(SimStatePlugin):
                     if conc_end & 0xfff:
                         conc_end = (conc_end & ~0xfff) + 0x1000
                     # TODO: figure out what permissions to use
-                    self.state.memory.map_region(conc_start, conc_end - conc_start, 7)
+                    try:
+                        self.state.memory.map_region(conc_start, conc_end - conc_start, 7)
+                    except SimMemoryError as e:
+                        if len(e.args) >= 2 and type(e.args[1]) is int:
+                            l.warning("The heap seems to have collided with another allocation. Be careful!")
+                            self.brk = e.args[1]
+                        else:
+                            raise
 
         return self.brk
 
@@ -364,7 +402,7 @@ class SimSystemPosix(SimStatePlugin):
             pass
 
         ideal = self._pick_fd()
-        self.state.solver.add(fd == ideal)
+        self.state.add_constraints(fd == ideal)
         if not self.state.solver.satisfiable():
             raise SimPosixError("Tried to do operation on symbolic but partially constrained file descriptor")
         fd = ideal
@@ -489,33 +527,6 @@ class SimSystemPosix(SimStatePlugin):
             oldmask
         )
 
-    @SimStatePlugin.memo
-    def copy(self, memo):
-        o = SimSystemPosix(
-                stdin=self.stdin.copy(memo),
-                stdout=self.stdout.copy(memo),
-                stderr=self.stderr.copy(memo),
-                fd={k: self.fd[k].copy(memo) for k in self.fd},
-                sockets={ident: tuple(x.copy(memo) for x in self.sockets[ident]) for ident in self.sockets},
-                socket_queue=self.socket_queue, # shouldn't need to copy this - should be copied before use.
-                                                # as a result, we must update the state of each socket before making
-                                                # copies.
-                argv=self.argv,
-                argc=self.argc,
-                environ=self.environ,
-                auxv=self.auxv,
-                tls_modules=self.tls_modules,
-                sigmask=self._sigmask,
-                pid=self.pid,
-                ppid=self.ppid,
-                uid=self.uid,
-                gid=self.gid,
-                brk=self.brk)
-        o.dev_fs = self.dev_fs.copy(memo)
-        o.proc_fs = self.proc_fs.copy(memo)
-        o._closed_fds = list(self._closed_fds)
-        return o
-
     def merge(self, others, merge_conditions, common_ancestor=None):
         for o in others:
             if len(self.fd) != len(o.fd):
@@ -546,10 +557,14 @@ class SimSystemPosix(SimStatePlugin):
                 common_sock = common_ancestor.sockets[ident]
             except (AttributeError, KeyError):
                 common_sock = None
-            merging_occurred |= self.sockets[ident].merge(
-                    [o.sockets[ident] for o in others],
+            merging_occurred |= self.sockets[ident][0].merge(
+                    [o.sockets[ident][0] for o in others],
                     merge_conditions,
-                    common_ancestor=common_sock)
+                    common_ancestor=common_sock[0])
+            merging_occurred |= self.sockets[ident][1].merge(
+                [o.sockets[ident][1] for o in others],
+                merge_conditions,
+                common_ancestor=common_sock[1])
 
         # pylint: disable=no-member
         # pylint seems to be seriously flipping out here for reasons I'm unsure of
@@ -598,4 +613,4 @@ class SimSystemPosix(SimStatePlugin):
 from angr.sim_state import SimState
 SimState.register_default('posix', SimSystemPosix)
 
-from ..errors import SimPosixError, SimSolverError, SimMergeError
+from ..errors import SimPosixError, SimSolverError, SimMergeError, SimMemoryError
