@@ -2,6 +2,7 @@ import binascii
 import functools
 import time
 import logging
+from typing import Type, TypeVar, overload, Any, Optional
 
 from claripy import backend_manager
 
@@ -10,7 +11,7 @@ from .sim_action_object import ast_stripping_decorator, SimActionObject
 
 l = logging.getLogger(name=__name__)
 
-#pylint:disable=unidiomatic-typecheck
+#pylint:disable=unidiomatic-typecheck,isinstance-second-argument-not-valid-type
 
 #
 # Timing stuff
@@ -48,7 +49,8 @@ def timed_function(f):
                 location = "unknown"
             lt.log(int((end-start)*10), '%s took %s seconds at %s', f.__name__, round(duration, 2), location)
 
-            if break_time >= 0 and duration > break_time:
+            if 0 <= break_time < duration:
+                #pylint: disable = import-outside-toplevel
                 import ipdb; ipdb.set_trace()
 
             return r
@@ -179,8 +181,8 @@ class SimSolver(SimStatePlugin):
     Any top-level variable of the claripy module can be accessed as a property of this object.
     """
     def __init__(self, solver=None, all_variables=None, temporal_tracked_variables=None, eternal_tracked_variables=None): #pylint:disable=redefined-outer-name
-        l.debug("Creating SimSolverClaripy.")
-        SimStatePlugin.__init__(self)
+        super().__init__()
+
         self._stored_solver = solver
         self.all_variables = [] if all_variables is None else all_variables
         self.temporal_tracked_variables = {} if temporal_tracked_variables is None else temporal_tracked_variables
@@ -250,6 +252,8 @@ class SimSolver(SimStatePlugin):
         """
         Given an AST, iterate over all the keys of all the BVS leaves in the tree which are registered.
         """
+        # pylint: disable=stop-iteration-return
+        # ??? wtf pylint
         reverse_mapping = {next(iter(var.variables)): k for k, var in self.eternal_tracked_variables.items()}
         reverse_mapping.update({next(iter(var.variables)): k for k, var in self.temporal_tracked_variables.items() if k[-1] is not None})
 
@@ -276,7 +280,11 @@ class SimSolver(SimStatePlugin):
             elif 'smtlib_abc' in backend_manager.backends._backends_by_name:
                 our_backend = backend_manager.backends.smtlib_abc
             else:
-                raise ValueError("Could not find suitable string solver!")
+                l.error("Cannot find a suitable string solver. Please ensure you have installed a string solver that "
+                        "angr supports, and have imported the corresponding solver backend in claripy. You can try "
+                        "adding \"from claripy.backends.backend_smtlib_solvers import *\" at the beginning of your "
+                        "script.")
+                raise ValueError("Cannot find a suitable string solver")
             if o.COMPOSITE_SOLVER in self.state.options:
                 self._stored_solver = claripy.SolverComposite(
                     template_solver_string=claripy.SolverCompositeChild(backend=our_backend, track=track)
@@ -405,7 +413,7 @@ class SimSolver(SimStatePlugin):
             return f
 
     def __dir__(self):
-        return sorted(set(dir(super(SimSolver, self)) + dir(claripy._all_operations) + dir(self.__class__)))
+        return sorted(set(dir(super()) + dir(claripy._all_operations) + dir(self.__class__)))
 
     #
     # Branching stuff
@@ -413,7 +421,14 @@ class SimSolver(SimStatePlugin):
 
     @SimStatePlugin.memo
     def copy(self, memo): # pylint: disable=unused-argument
-        return SimSolver(solver=self._solver.branch(), all_variables=self.all_variables, temporal_tracked_variables=self.temporal_tracked_variables, eternal_tracked_variables=self.eternal_tracked_variables)
+        c = super().copy(memo)
+
+        c._stored_solver = self._solver.branch()
+        c.all_variables = self.all_variables
+        c.temporal_tracked_variables = self.temporal_tracked_variables
+        c.eternal_tracked_variables = self.eternal_tracked_variables
+
+        return c
 
     @error_converter
     def merge(self, others, merge_conditions, common_ancestor=None): # pylint: disable=W0613
@@ -655,8 +670,9 @@ class SimSolver(SimStatePlugin):
     # And some convenience stuff
     #
 
+    T = TypeVar('T', int, bytes)
     @staticmethod
-    def _cast_to(e, solution, cast_to):
+    def _cast_to(e, solution, cast_to: Type[T]) -> T:
         """
         Casts a solution for the given expression to type `cast_to`.
 
@@ -687,7 +703,16 @@ class SimSolver(SimStatePlugin):
 
         return solution
 
-    def eval_upto(self, e, n, cast_to=None, **kwargs):
+
+    @overload
+    # pylint: disable=no-self-use
+    def eval_upto(self, e, n: int, cast_to: None = ..., **kwargs) -> Any: ...
+    CastTarget = TypeVar("CastTarget")
+    @overload
+    # pylint: disable=no-self-use, undefined-variable
+    def eval_upto(self, e, n: int, cast_to: CastTarget = ..., **kwargs) -> CastTarget: ...
+
+    def eval_upto(self, e, n, cast_to = None, **kwargs):
         """
         Evaluate an expression, using the solver if necessary. Returns primitives as specified by the `cast_to`
         parameter. Only certain primitives are supported, check the implementation of `_cast_to` to see which ones.
@@ -709,7 +734,15 @@ class SimSolver(SimStatePlugin):
             raise SimUnsatError('Not satisfiable: %s, expected up to %d solutions' % (e.shallow_repr(), n))
         return cast_vals
 
-    def eval(self, e, **kwargs):
+    @overload
+    # pylint: disable=no-self-use
+    def eval(self, e, cast_to: None = ..., **kwargs) -> Any: ...
+
+    # pylint: disable=no-self-use,undefined-variable
+    @overload
+    def eval(self, e, cast_to: CastTarget = ..., **kwargs) -> CastTarget: ...
+
+    def eval(self, e, cast_to: Optional[CastTarget] = None, **kwargs):
         """
         Evaluate an expression to get any possible solution. The desired output types can be specified using the
         `cast_to` parameter. `extra_constraints` can be used to specify additional constraints the returned values
@@ -721,7 +754,11 @@ class SimSolver(SimStatePlugin):
         :return:
         """
         # eval_upto already throws the UnsatError, no reason for us to worry about it
-        return self.eval_upto(e, 1, **kwargs)[0]
+        concrete_val = _concrete_value(e)
+        if concrete_val is not None:
+            return self._cast_to(e, concrete_val, cast_to)
+
+        return self.eval_upto(e, 1, cast_to = cast_to,  **kwargs)[0]
 
     def eval_one(self, e, **kwargs):
         """
